var AlbumView = {
  initiated : false,
  container : null,
  img_container : null,
  caption : null,
  keepFocus : true,
  updatedSelection : false,
  current: 0, target: 0, timer: 0,
  middle : 0, biggest : 0, max : 0,
  MAX_NODES: 6,
  lastOffsets : new Array(),
  itemNodes : new Array(),
  itemImages : new Array(),
  itemProps : new Array(),
  canRefresh : true,
  STEP_INTERVAL : 50,
  SHOW_REFLECTION : false,
  init : function(aContainer, aImgContainer, aCaption){
    this.canvas = document.getElementById("albumcanvas");
    this.container = document.getElementById("albumview-container");
    this.img_container = document.getElementById("albumview-image-container");
    this.caption = document.getElementById("albumview-caption");
    this.container.addEventListener('click',this.displayClick,false);
    this.container.addEventListener('dblclick',this.displayDblClick,false);
    this.container.addEventListener('DOMMouseScroll', this.wheel, false);
    this.container.addEventListener('keypress',this.handleKeypress,true);
    //this.container.addEventListener('click',this.containerClick,false);
    this.initiated = true;
  },
  uninit : function(){
    this.container.removeEventListener('click',this.displayClick,false);
    this.container.removeEventListener('dblclick',this.displayDblClick,false);
    this.container.removeEventListener('DOMMouseScroll', this.wheel, false);
    this.container.removeEventListener('keypress',this.handleKeypress,true);
    //this.container.removeEventListener('click',this.containerClick,false);
    this.initiated = false;
  },
  /* called on start and on refresh
    caches unchanging values*/
  refresh : function(event) {
    //dump("refresh!");
    //return;
    var self = AlbumView;
    self.canRefresh = false;
    var images_width = self.container.boxObject.width;//self.img_container.parentNode.parentNode.parentNode.boxObject.width;
    self.middle = images_width * 0.5;
    self.biggest = images_width * 0.3;
    if(self.biggest>300)self.biggest=300;
    images_height = Math.ceil(1.1 * self.biggest - .125 * self.middle);
    var lblHeight = self.caption.boxObject.height;
    var curHeight = self.container.boxObject.height - lblHeight;
    var availableHeight = self.container.parentNode.boxObject.height - lblHeight;
    if (availableHeight<images_height){
      images_height = availableHeight;
      if (images_height < self.biggest)
        self.biggest = images_height;
    }

    self.setSize(self.img_container,images_width,images_height);
    if (curHeight>images_height){
      //dump("resize container: "+curHeight+"|"+images_height+"\n");
      self.container.height = images_height+lblHeight;
    }
    self.slope = (images_height)/(images_width/2);
    self.height = images_height;
    self.width = self.middle*5/4;
    self.real_width = images_width;

    /* Display images in current order */
    self.moveTo(self.current);
    self.canRefresh = true;
  },
  step : function() {
    var self = AlbumView;
    if (self.target < self.current-1 || self.target > self.current+1)	{
        self.moveTo(self.current + (self.target-self.current)/3);
        setTimeout(self.step, self.STEP_INTERVAL);
        self.timer = 1;
    } else {
      self.timer = 0;
      self.current = self.target;
      var targetIndex = -AlbumView.target/150
      if (AlbumViewUtils && AlbumViewUtils.selectIndex)
        AlbumViewUtils.selectIndex(self.itemNodes[targetIndex]);
    }
  },
  glideToIndex : function(index){
    var target = index * -150;
    AlbumView.glideTo(target);
  },
  glideTo : function(aTarget,updateSelection) {
    /* Animate gliding to new x position */
    var self = AlbumView;
    self.target = aTarget;
    var targetIndex = -self.target/150;
    var currentIndex = -self.current/150;
    if (targetIndex>=self.max){
      if (currentIndex<self.max-1)
        self.target = -(self.max-1)*150;
    }
    if (self.timer == 0 && targetIndex<self.max && targetIndex!=currentIndex){
      setTimeout(self.step, 50);
      self.timer = 1;
    }
    if (targetIndex<self.max){
      /* Display new caption */
      var caption = self.itemProps[targetIndex].caption;
      self.caption.value = caption;
    }
  },
  /* main part of the transition */
  moveTo : function(x) {
    //dump("moveto: "+x+"\n");
    try{
    var self = AlbumView;
    self.current = x;
    var targetIndex = Math.ceil(-self.target / 150);
    var currentIndex = Math.ceil(-self.current / 150);
    var start=targetIndex-self.MAX_NODES;
    if (start<0)
      start=0;
    var end=targetIndex+self.MAX_NODES;
    if (end>self.max - 1)
      end = self.max - 1;
    //dump("start: "+start+" tindex: "+targetIndex+" end: "+end+"\n");
    var ctx = self.img_container.getContext("2d");
    ctx.clearRect(0,0,self.real_width,self.height);

    function moveLoop(index,increasing){
      self.computeProperties(x,self.itemProps[index]);
      var success = self.drawImage(index, currentIndex, targetIndex);
      //if (success)
      //  dump("index: "+index+"\n");
      if (increasing) x += 150;
      else x -= 150;
    }
    x = x + 150 * start;
    for (var index = start; index < targetIndex; index++){
      moveLoop(index,true);
    }
    x = self.current + end * 150;
    for (var index = end; index>=targetIndex; index--){
      moveLoop(index, false);
    }
    }catch(e){Components.utils.reportError("MOVETO: "+e);}
  },
  computeProperties : function(x, props){
    var z = Math.sqrt(10000 + Math.pow(x,2))+100;
    props.width = Math.floor(200 / z * this.biggest);
    //props.height = Math.floor(200 / (Math.sqrt(10000 + Math.pow(x,2))+100) * this.biggest -
    //               25 / (Math.sqrt(10000 + Math.pow(x,2))+100) * this.width);
    props.height = Math.floor(220 / (Math.sqrt(10000 + Math.pow(x,2))+100) * this.biggest -
                   25 / (Math.sqrt(10000 + Math.pow(x,2))+100) * this.width);
    props.left = Math.floor((x / (Math.sqrt(10000 + Math.pow(x,2))+300))*this.width + this.middle - props.width/2);
    props.top = Math.floor((this.height - props.height)/2);
  },
  /* returns true if it should be drawn, false if it shouldn't(out of bounds)*/
  drawImage : function(index, currentIndex, targetIndex){
    try{
      //dump("draw image: "+index+"\n");
    var self = AlbumView;
    if (!currentIndex)
      currentIndex = -self.current / 150;
    if (!targetIndex)
      targetIndex = -self.target / 150;
    var offset = index - targetIndex;
    var props = self.itemProps[index];
    if (props.left==null){
      self.computeProperties(offset * -150,props);
    }
    if (props.left<0 || props.left+props.width>self.real_width){
      return false;
    }
    var ctx = self.img_container.getContext("2d");
    var img = self.itemImages[index];
    if (img.complete){
      var hoffset = 0, toffset = 0;
      if (ctx.transform && offset!=0 && false){/* FF3 only*/
        if (!self._tempCanvas){
          self._tempCanvas = document.createElementNS("http://www.w3.org/1999/xhtml","canvas");
        }
        var angle, tanA;
        if (offset < 0){
          angle = -Math.PI / 18;
          tanA = Math.tan(angle);
          hoffset = Math.abs(props.height * tanA * 2);
          toffset = hoffset;
        }
        else{
          angle = Math.PI / 18;
          tanA = Math.tan(angle);
          hoffset = props.height * tanA * 2;
        }
        self.setSize(self._tempCanvas,props.width,props.height+hoffset);
        var ctx2 = self._tempCanvas.getContext("2d");
        ctx2.clearRect(0,0,img.width,props.height+hoffset);
        ctx2.transform(1, tanA, 0, 1, 0, 0);
        ctx2.drawImage(img,0,toffset,props.width,props.height);
        img = self._tempCanvas;
      }

      ctx.drawImage(img,
                    props.left,
                    props.top-toffset/2,
                    props.width,
                    props.height+hoffset);

      if (self.SHOW_REFLECTION){
        ctx.drawImage(img, 0, 0, img.width, img.height,
                      props.left,
                      props.top+props.height,
                      props.width,
                      Math.floor(props.height/3));
        var lineargradient = ctx.createLinearGradient(0,props.top+props.height,0,props.top+props.height+Math.floor(props.height/3));//Math.floor(props.height/3));

        lineargradient.addColorStop(0,'rgba(0,0,0,.3)');
        lineargradient.addColorStop(.5,'rgba(30,30,30,.5)');
        lineargradient.addColorStop(1,'rgba(0,0,0,1)');
        ctx.fillStyle = lineargradient;
        ctx.fillRect(props.left,props.top+props.height,props.width,Math.floor(props.height/3));

      }
    }
    self.lastOffsets[index] = offset;
    //dump("finish draw image\n");
    }catch(e){Components.utils.reportError(e);}
    return true;
  },
  _getClickTarget : function(event){
    var x = (event.clientX - this.container.boxObject.x);//event.layerX;
    var y = (event.clientY - this.container.boxObject.y);//event.layerY;
    var targetIndex = -this.target / 150;
    //dump("x: "+x+" y: "+y+" middle: "+this.middle+" target: "+targetIndex+"\n");
    //dump("client: "+event.clientX+" realx: "+(event.clientX - this.container.boxObject.x)+"\n");
    /* less than or equal to targetIndex */
    if (x<this.middle){
      for (var i = Math.ceil(targetIndex); i>=0; i--){
        var props = this.itemProps[i];
        if (x>=props.left && x<=props.left + props.width){
          return i;
        }
      }
    }
    else{
      for (var i = Math.floor(targetIndex); i< this.max; i++){
        var props = this.itemProps[i];
        if (x>=props.left && x<=props.left + props.width){
          return i;
        }
      }
    }
    return -1;
  },
  displayClick : function(event){
    var index = AlbumView._getClickTarget(event);
    if (index == -1) return;
    if (event.button == 1)
      AlbumViewUtils.performAction(AlbumView.itemNodes[index],event);
    else
      AlbumView.glideTo(index * -150,true);
  },
  displayDblClick : function(event){
    var index = AlbumView._getClickTarget(event);
    if (index != -1){
      AlbumViewUtils.performAction(AlbumView.itemNodes[index],event);
    }
  },
  /* JavaScript mouse wheel & keyboard support */
  handle : function(delta) {
    if  (delta > 0) {
      if(AlbumView.target != 0)
        AlbumView.target += 150;
    } else if(-AlbumView.current/150 < AlbumView.max) {
        AlbumView.target = AlbumView.target - 150;
    }
    AlbumView.glideTo(AlbumView.target,true);
  },
  /* JavaScript mouse wheel support */
  wheel : function(event){
    var delta = -event.detail/3;
    AlbumView.handle(delta);
    event.preventDefault();
  },
  handleKeypress : function(event){
    var KeyEvent = Ci.nsIDOMKeyEvent;
    switch(event.keyCode){
      case KeyEvent.DOM_VK_LEFT:
        AlbumView.handle(1)
        break;
      case KeyEvent.DOM_VK_RIGHT:
        AlbumView.handle(-1);
        break;
      //TODO: This won't work since it isn't focusable anymore
      case KeyEvent.DOM_VK_RETURN:
      case KeyEvent.DOM_VK_ENTER:
        if (AlbumViewUtils && AlbumViewUtils.performAction){
          var targetIndex = -AlbumView.target/150;
          AlbumViewUtils.performAction(AlbumView.itemNodes[targetIndex]);
        }
    }
  },
  createImages : function(aImages){
    dump("create images\n");
    try{
    var totalImages = aImages;
    AlbumView.max = totalImages.length;
    AlbumView.itemNodes = new Array(AlbumView.max);
    AlbumView.itemImages = new Array(AlbumView.max);
    AlbumView.itemProps = new Array(AlbumView.max);
    /* remove any remaining children */
    dump("max: "+totalImages.length+"\n");
    for (var i = AlbumView.max-1; i>=0; i--){
      var src = totalImages[i];
      AlbumView.lastOffsets[i]=i;
      AlbumView.itemNodes[i]=src.node;
      AlbumView.itemProps[i] = {caption : src.caption};
      AlbumView.itemImages[i]=new Image();
      AlbumView.itemImages[i].setAttribute("onload",
          "AlbumView.imageLoaded(event,"+i+");");
      //addEventListener("load",
      //          function(event){AlbumView.imageLoaded(event,i)},
      //          false);
      AlbumView.itemImages[i].src = src.image;
      //dump("completed: "+AlbumView.itemImages[i].complete+"\n");
    }
    }catch(e){Components.utils.reportError(e);}
  },
  imageLoaded : function(event,idx){
    var targetIndex = -this.target / 150, canDraw = true;
    if (idx>targetIndex){
      for (var i = idx; i>=targetIndex && canDraw; i--){
        canDraw = AlbumView.drawImage(i);
      }
    }
    else if (idx<targetIndex){
      for (var i = idx; i<=targetIndex && canDraw; i++){
        canDraw = AlbumView.drawImage(i);
      }
    }
  },
  setSize : function(ele,width,height){
    ele.setAttribute("height",height);
    ele.setAttribute("width",width);
    ele.width=width;
    ele.height=height;
    //ele.style.maxWidth = width+"px";
    //ele.style.maxHeight = height+"px";
    //ele.setAttribute("style","max-height: "+height+"px; max-width: "+width+"px");
  }
}
